//=============================================================================

#include <iostream>
#include <limits>
#include "computerfff.hpp"
#include "computerlff.hpp"
#include "human.hpp"
#include "tictactoe.hpp"
using namespace std;

//-----------------------------------------------------------------------------

bool play(Player& pl, TicTacToe& ttt)
{
   // 1. Spieler nach seinem Zug fragen
   Move m = pl.askForMove(ttt);

   // 2. Ausgabe Zug
   cout << "\n" << pl.getName() << " macht Zug : " << m << "\n\n";

   // 3. Zug wird gesetzt
   ttt.set(m);

   // 4. Spiel ausgeben
   cout << ttt << '\n';

   // 5. berprfen, ob Spieler gewonnen hat: ja => Spielende mit Meldung
   if (ttt.hasWin(m.getGameColor()))
   {
      cout << "Gratulation - " << pl.getName() << " hat gewonnen - juhuuu...\n";
      return true;
   }

   // 6. berprfen, ob das Spiel am Ende ist: ja => remis => Spielende mit Meldung
   if (ttt.isFull())
   {
      cout << "Spiel ist Remis ausgegangen...\n";
      return true;
   }

   return false;
}

//-----------------------------------------------------------------------------

PlayerPtr selectPlayer(GameColor gc)
{
   for (int i;;)
   {
      cout << "Welcher Spieler (" << gc << ") soll erzeugt werden ?\n1: Mensch\n2: Computer FFF\n3: Computer LFF\n\n> ";
      cin >> i;
      if (cin.fail())
      {
         cin.clear();
         cin.ignore(numeric_limits<streamsize>::max(), '\n');
         cout << "Keine Zahl eingegeben - bitte nochmal versuchen\n\n";
         continue;
      }
      switch (i)
      {
      case 1:
         return make_unique<Human>(gc);
      case 2:
         return make_unique<ComputerFFF>(gc);
      case 3:
         return make_unique<ComputerLFF>(gc);
      default:
         cout << "Fehlerhafte Zahl eingegeben - bitte nochmal versuchen\n\n";
      }
   }
}

//-----------------------------------------------------------------------------

int main()
{
   cout << "WS 2024/25 Tic-Tac-Toe 3\n\n";

   // 0. Player erzeugen
   PlayerPtr pl1 = selectPlayer(GameColor::white);
   PlayerPtr pl2 = selectPlayer(GameColor::black);

   // 1. Spiel erzeugen
   TicTacToe ttt;
   
   // 2. Spiel ausgeben
   cout << ttt << '\n';

   for (;;)
   {
      if (play(*pl1, ttt)) break;
      if (play(*pl2, ttt)) break;
   }
}
