#ifndef PLAYER_HPP
#define PLAYER_HPP

#include "manager.hpp"
#include "move.hpp"
#include "tictactoe.hpp"
#include <memory>
#include <string>

class Player
{
public:
   Player(GameColor gc) : mGameColor{ gc } {}

   virtual ~Player() = default;

   virtual Move askForMove(const TicTacToe&) = 0;

   virtual std::string getName() const = 0;

   GameColor getGameColor() const { return mGameColor; }

private:
   GameColor mGameColor;
};

using PlayerPtr = std::unique_ptr<Player>;

class PlayerCreator
{
public:
   virtual std::string getPlayerName() const = 0;

   virtual PlayerPtr createPlayer(GameColor gc) const = 0;
};

using PlayerCreatorPtr = std::unique_ptr<PlayerCreator>;

template<class T> class PlayerCreatorCreator
{
public:
   PlayerCreatorCreator()
   {
      Manager::mCreators.push_back(std::make_unique<T>());
   }
};

#endif
